==> tutti-ui-swing-2.2-sources.jar: fr/ifremer/tutti/ui/swing/util/action/TuttiUIAction.java <==
package fr.ifremer.tutti.ui.swing.util.action;

/*
 * #%L
 * Tutti :: UI
 * $Id: TuttiUIAction.java 715 2013-04-01 20:46:21Z tchemit $
 * $HeadURL: http://svn.forge.codelutin.com/svn/tutti/tags/tutti-2.2/tutti-ui-swing/src/main/java/fr/ifremer/tutti/ui/swing/util/action/TuttiUIAction.java $
 * %%
 * Copyright (C) 2012 - 2013 Ifremer
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-3.0.html>.
 * #L%
 */

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.swing.AbstractAction;
import javax.swing.AbstractButton;
import javax.swing.Icon;
import javax.swing.SwingUtilities;
import java.awt.event.ActionEvent;
import java.util.concurrent.CancellationException;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * Abstract tutti ui action which launch a {@link AbstractTuttiAction}.
 *
 * @author tchemit <chemit@codelutin.com>
 * @since 1.0.2
 */
public class TuttiUIAction<A extends AbstractTuttiAction> extends AbstractAction {

    private static final long serialVersionUID = 1L;

    /** Logger. */
    private static final Log log = LogFactory.getLog(TuttiUIAction.class);

    public static final ExecutorService waitingThread =
            Executors.newSingleThreadExecutor();

    private static final String LOGIC_ACTION = "logicAction";

    private final Object lock = new Object();

    private boolean wait;

    public TuttiUIAction(AbstractButton button, A action) {

        putValue(LOGIC_ACTION, action);

        // fill the ui action from the button
        setActionKey(action.getClass().getName());
        if (button != null) {
            setActionIcon(button.getIcon());
            setActionName(button.getText());
            setActionDescription(button.getToolTipText());
            setActionMnemonic(button.getMnemonic());
        }
    }

    public void launchActionAndWait() {
        wait = true;
        actionPerformed(null);
        lock();
    }


    @Override
    public final void actionPerformed(final ActionEvent event) {

        if (log.isInfoEnabled()) {
            log.info("Task [" + this + "] starting");
        }

        // prepare action
        boolean doAction;

        A action = getLogicAction();

        // reset status message
        action.sendMessage("");

        try {
            doAction = action.prepareAction();
        } catch (Exception e) {
            action.releaseAction();
            throw TuttiActionException.propagateError(action, e);
        }

        if (doAction) {

            final TuttiActionSwingWorker<A> worker =
                    new TuttiActionSwingWorker<A>(action);

            SwingUtilities.invokeLater(new Runnable() {
                @Override
                public void run() {

                    // make ui busy
                    worker.updateBusyState(true);

                }
            });

            if (log.isInfoEnabled()) {
                log.info("Before execute of action " + action);
            }

            // perform and release action
            worker.execute();

            // wait until action is done
            waitingThread.execute(
                    new Runnable() {
                        @Override
                        public void run() {

                            A action = getLogicAction();
                            try {
                                try {
                                    worker.get();
                                } catch (ExecutionException e) {
                                    // don't care .
                                } catch (CancellationException e) {
                                    // dont care ?
                                } catch (InterruptedException e) {
                                    // don't care ?
                                }
                                if (log.isInfoEnabled()) {
                                    log.info("After execute of action " + action + " (worker done? " + worker.isDone() + ")");
                                }

                                if (worker.isFailed()) {

                                    throw TuttiActionException.propagateError(action, worker.getError());
                                }
                            } finally {
                                unlock();
                            }
                        }
                    }
            );

        } else {

            try {
                // release action
                action.releaseAction();

            } finally {
                unlock();
            }
        }
    }

    public void setActionIcon(Icon actionIcon) {
        putValue(SMALL_ICON, actionIcon);
        putValue(LARGE_ICON_KEY, actionIcon);
    }

    public void setActionKey(String actionKey) {
        putValue(ACTION_COMMAND_KEY, actionKey);
    }

    public void setActionName(String actionName) {
        putValue(NAME, actionName);
    }

    public void setActionDescription(String actionDescription) {
        putValue(SHORT_DESCRIPTION, actionDescription);
        getLogicAction().setActionDescription(actionDescription);
    }

    public void setActionMnemonic(int key) {
        putValue(MNEMONIC_KEY, key);
    }

    public A getLogicAction() {
        return (A) getValue(LOGIC_ACTION);
    }

    protected void lock() {
        if (wait) {
            synchronized (lock) {
                try {

                    lock.wait();

                } catch (InterruptedException e) {
                    throw TuttiActionException.propagateError(getLogicAction(), e);
                } finally {
                    wait = false;
                }
            }
        }
    }

    protected void unlock() {
        if (wait) {
            synchronized (lock) {
                lock.notifyAll();
            }
        }
    }
}

==> tutti-ui-swing-2.3-sources.jar: fr/ifremer/tutti/ui/swing/util/action/TuttiUIAction.java <==
package fr.ifremer.tutti.ui.swing.util.action;

/*
 * #%L
 * Tutti :: UI
 * $Id: TuttiUIAction.java 983 2013-05-22 12:28:46Z tchemit $
 * $HeadURL: http://svn.forge.codelutin.com/svn/tutti/tags/tutti-2.3/tutti-ui-swing/src/main/java/fr/ifremer/tutti/ui/swing/util/action/TuttiUIAction.java $
 * %%
 * Copyright (C) 2012 - 2013 Ifremer
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-3.0.html>.
 * #L%
 */

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.swing.AbstractAction;
import javax.swing.AbstractButton;
import javax.swing.Icon;
import javax.swing.SwingUtilities;
import java.awt.event.ActionEvent;
import java.util.concurrent.CancellationException;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * Abstract tutti ui action which launch a {@link AbstractTuttiAction}.
 *
 * @author tchemit <chemit@codelutin.com>
 * @since 1.0.2
 */
public class TuttiUIAction<A extends AbstractTuttiAction> extends AbstractAction {

    private static final long serialVersionUID = 1L;

    /** Logger. */
    private static final Log log = LogFactory.getLog(TuttiUIAction.class);

    public static final ExecutorService waitingThread =
            Executors.newSingleThreadExecutor();

    private static final String LOGIC_ACTION = "logicAction";

    private final Object lock = new Object();

    private boolean wait;

    public TuttiUIAction(AbstractButton button, A action) {

        putValue(LOGIC_ACTION, action);

        // fill the ui action from the button
        setActionKey(action.getClass().getName());
        if (button != null) {
            setActionIcon(button.getIcon());
            setActionName(button.getText());
            setActionDescription(button.getToolTipText());
            setActionMnemonic(button.getMnemonic());
        }
    }

    public void launchActionAndWait() {
        wait = true;
        actionPerformed(null);
        lock();
    }


    @Override
    public final void actionPerformed(final ActionEvent event) {

        if (log.isInfoEnabled()) {
            log.info("Task [" + getLogicAction() + "] starting");
        }

        // prepare action
        boolean doAction;

        A action = getLogicAction();

        // reset status message
        action.sendMessage("");

        try {
            doAction = action.prepareAction();
        } catch (Exception e) {
            action.releaseAction();
            throw TuttiActionException.propagateError(action, e);
        }

        if (doAction) {

            final TuttiActionSwingWorker<A> worker =
                    new TuttiActionSwingWorker<A>(action);

            SwingUtilities.invokeLater(new Runnable() {
                @Override
                public void run() {

                    // make ui busy
                    worker.updateBusyState(true);

                }
            });

            if (log.isDebugEnabled()) {
                log.debug("Before execute of action " + action);
            }

            // perform and release action
            worker.execute();

            // wait until action is done
            waitingThread.execute(
                    new Runnable() {
                        @Override
                        public void run() {

                            A action = getLogicAction();
                            try {
                                try {
                                    worker.get();
                                } catch (ExecutionException e) {
                                    // don't care .
                                } catch (CancellationException e) {
                                    // dont care ?
                                } catch (InterruptedException e) {
                                    // don't care ?
                                }
                                if (log.isDebugEnabled()) {
                                    log.debug("After execute of action " + action + " (worker done? " + worker.isDone() + ")");
                                }

                                if (worker.isFailed()) {

                                    throw TuttiActionException.propagateError(action, worker.getError());
                                }
                            } finally {
                                unlock();
                            }
                        }
                    }
            );

        } else {

            try {
                // release action
                action.releaseAction();

            } finally {
                unlock();
            }
        }
    }

    public void setActionIcon(Icon actionIcon) {
        putValue(SMALL_ICON, actionIcon);
        putValue(LARGE_ICON_KEY, actionIcon);
    }

    public void setActionKey(String actionKey) {
        putValue(ACTION_COMMAND_KEY, actionKey);
    }

    public void setActionName(String actionName) {
        putValue(NAME, actionName);
    }

    public void setActionDescription(String actionDescription) {
        putValue(SHORT_DESCRIPTION, actionDescription);
        getLogicAction().setActionDescription(actionDescription);
    }

    public void setActionMnemonic(int key) {
        putValue(MNEMONIC_KEY, key);
    }

    public A getLogicAction() {
        return (A) getValue(LOGIC_ACTION);
    }

    protected void lock() {
        if (wait) {
            synchronized (lock) {
                try {

                    lock.wait();

                } catch (InterruptedException e) {
                    throw TuttiActionException.propagateError(getLogicAction(), e);
                } finally {
                    wait = false;
                }
            }
        }
    }

    protected void unlock() {
        if (wait) {
            synchronized (lock) {
                lock.notifyAll();
            }
        }
    }
}
